/**
 * Example config
 */

var config = {
  endpoint: 'http://localhost:3000/access', 
  resource_id: '3c', 
  resource_secret: '41d442fcd3a2cafc0e14'
};


/**
 * Example app
 */

// app dependencies
var express = require('express')
  , authorize = require('./index')(config)
  , app = express()
  ;

// app config
app.configure(function () {
  app.set('port', 3001);
  app.use(express.bodyParser());
  app.use(app.router);

  // Error handler
  app.use(function (err, req, res, next) {
    var error = (err.errors)
      ? { errors: err.errors }
      : { error: err.message, error_description: err.description };
    res.send(err.statusCode || 500, error);
  });
});

// Protected route
app.get('/protected', authorize, function (req, res) {
  res.send('protected resource');
});


if (!module.parent) {
  app.listen(app.settings.port, function () {
    console.log(
        'OAuth2Resource is running on port ' + app.settings.port
    );   
  });
}



//
//    // without access token
//    $ curl -v localhost:3001/protected
//
//    // without client id
//    $ curl -v localhost:3001/protected -H 'Authorization: Bearer 75a07e0a7ba7b5978319'
//    
//    // without scope
//    $ curl -v 'localhost:3001/protected?client_id=2b' -H 'Authorization: Bearer 75a07e0a7ba7b5978319'
//
//    // with unknown access token
//    $ curl -v 'localhost:3001/protected?client_id=2b&scope=limited' -H 'Authorization: Bearer unknown'
//    
//    // with mismatching client
//    $ curl -v 'localhost:3001/protected?client_id=wrong&scope=limited' -H 'Authorization: Bearer 75a07e0a7ba7b5978319'
//
//    // with insufficient scope
//    $ curl -v 'localhost:3001/protected?client_id=2b&scope=insufficient' -H 'Authorization: Bearer 75a07e0a7ba7b5978319'
//
//    // valid request
//    $ curl -v 'localhost:3001/protected?client_id=2b&scope=limited' -H 'Authorization: Bearer 75a07e0a7ba7b5978319'
//


//hyperquest (substack)
//
//app.get('/whatever', authorize, function (req, res) {
//  var user_id = req.token.user_id
//  db.whatevers.find({ user_id: user_id }, function (err, docs) {...})
//});





